==> simplecloud-launcher-2.1.0-sources.jar: eu/thesimplecloud/launcher/external/module/handler/RecursiveDependencyChecker.kt <==
/*
 * MIT License
 *
 * Copyright (C) 2020 The SimpleCloud authors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package eu.thesimplecloud.launcher.external.module.handler

import eu.thesimplecloud.launcher.external.module.LoadedModuleFileContent

/**
 * Created by IntelliJ IDEA.
 * Date: 26.11.2020
 * Time: 17:33
 * @author Frederick Baier
 */
class RecursiveDependencyChecker(
        private val moduleToCheck: LoadedModuleFileContent,
        private val dependencyGetFunction: (LoadedModuleFileContent) -> Collection<LoadedModuleFileContent>
) {

    fun hasRecursiveDependencies(): Boolean {
        val allDependencies = collectAllDependencies(moduleToCheck)
        return allDependencies.any { doesDependencyDependOnModuleToCheck(it) }
    }

    private fun doesDependencyDependOnModuleToCheck(dependency: LoadedModuleFileContent): Boolean {
        return dependency.content.dependsOrSoftDependsOn(moduleToCheck.content)
    }

    fun collectAllDependencies(module: LoadedModuleFileContent, list: MutableList<LoadedModuleFileContent> = ArrayList()): MutableList<LoadedModuleFileContent> {
        if (list.contains(module))
            return list
        list.add(module)

        val dependencies = dependencyGetFunction(module)
        dependencies.forEach { collectAllDependencies(it, list) }
        return list
    }

}

==> simplecloud-launcher-2.8.0-pre.2-sources.jar: eu/thesimplecloud/launcher/external/module/handler/RecursiveDependencyChecker.kt <==
/*
 * MIT License
 *
 * Copyright (C) 2020-2022 The SimpleCloud authors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package eu.thesimplecloud.launcher.external.module.handler

import eu.thesimplecloud.launcher.external.module.LoadedModuleFileContent

/**
 * Created by IntelliJ IDEA.
 * Date: 26.11.2020
 * Time: 17:33
 * @author Frederick Baier
 */
class RecursiveDependencyChecker(
    private val moduleToCheck: LoadedModuleFileContent,
    private val dependencyGetFunction: (LoadedModuleFileContent) -> Collection<LoadedModuleFileContent>
) {

    fun hasRecursiveDependencies(): Boolean {
        val allDependencies = collectAllDependencies(moduleToCheck)
        return allDependencies.any { doesDependencyDependOnModuleToCheck(it) }
    }

    private fun doesDependencyDependOnModuleToCheck(dependency: LoadedModuleFileContent): Boolean {
        return dependency.content.dependsOrSoftDependsOn(moduleToCheck.content)
    }

    fun collectAllDependencies(
        module: LoadedModuleFileContent,
        list: MutableList<LoadedModuleFileContent> = ArrayList()
    ): MutableList<LoadedModuleFileContent> {
        if (list.contains(module))
            return list
        list.add(module)

        val dependencies = dependencyGetFunction(module)
        dependencies.forEach { collectAllDependencies(it, list) }
        return list
    }

}
